package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import java.util.Arrays;
import java.util.List;

public class ExampleMain {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) {
        try {
            persistEmployees();
            getNameSalaryPairsAsObjectArray();
            getNameSalaryPairsViaConstructorExpression();

        } finally {
            entityManagerFactory.close();
        }
    }

    public static void persistEmployees() {
        Employee employee1 = Employee.create("Diana", 2000, "IT");
        Employee employee2 = Employee.create("Rose", 3500, "Admin");
        Employee employee3 = Employee.create("Denise", 2500, "Admin");
        Employee employee4 = Employee.create("Mike", 4000, "IT");
        Employee employee5 = Employee.create("Linda", 4500, "Sales");
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(employee1);
        em.persist(employee2);
        em.persist(employee3);
        em.persist(employee4);
        em.persist(employee5);
        em.getTransaction().commit();

        System.out.println("-- employee persisted --");
        em.createQuery("Select e from Employee e")
          .getResultList()
          .forEach(System.out::println);
        em.close();
    }

    private static void getNameSalaryPairsAsObjectArray() {
        System.out.println("-- Employee name salary pair as Object[] --");
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createQuery("SELECT e.name, e.salary FROM Employee e");
        List<Object[]> resultList = query.getResultList();
        resultList.forEach(arr -> System.out.println(Arrays.toString(arr)));
        em.close();
    }

    private static void getNameSalaryPairsViaConstructorExpression() {
        System.out.println("-- Employee name salary pair as EmployeeNameSalaryPair --");
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query =
                em.createQuery("SELECT NEW com.logicbig.example.EmployeeNameSalaryPair(e.name, e.salary)"
                + " FROM Employee e");
        List<EmployeeNameSalaryPair> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }
}